 src/main_hlayer.d  src/main_hlayer.o:  src/main_hlayer.cc geometry/Sphere.h \
  geometry/AGeometricObject.h \
  util/vec3.h util/vec3.h src/InsertGenerator2D.h src/AGenerator2D.h \
  src/AVolume2D.h src/AVolume.h \
  geometry/Line2D.h \
  src/MNTable2D.h \
  src/MNTCell.h \
  geometry/Sphere.h geometry/LineSegment2D.h geometry/Line2D.h \
  src/BoxWithLines2D.h
