 src/main_sbx.d  src/main_sbx.o:  src/main_sbx.cc \
  util/vec3.h \
  geometry/Sphere.h geometry/Line2D.h geometry/LineSegment2D.h \
  util/vec3.h \
  geometry/Plane.h util/vec3.h geometry/AGeometricObject.h \
 
