 src/main_gouge.d  src/main_gouge.o:  src/main_gouge.cc geometry/Sphere.h \
  geometry/AGeometricObject.h \
  util/vec3.h util/vec3.h src/CircMNTable3D.h src/MNTable3D.h \
  src/MNTCell.h \
  geometry/Sphere.h geometry/Plane.h src/InsertGenerator3D.h \
  src/AGenerator3D.h src/AVolume3D.h src/AVolume.h \
  src/granular_gouge.h \
 
